"""Awesome package."""
import logging
import pickle
import re
from pathlib import Path
from kanjize import int2kanji
from dataclasses import dataclass

DIR_PATH = Path(__file__).parent
pkl_name = 'prefecture2city2street.pkl'
logger = logging.getLogger(__name__)
handler = logging.StreamHandler()

logger.addHandler(handler)

logger.setLevel(logging.INFO)

with open(str(DIR_PATH / pkl_name), 'rb') as f:
    prefecture2city = pickle.load(f)

__number_pat = re.compile(r'(\d+)')


@dataclass
class ParsedAddress:
    prefecture: str = ''
    city: str = ''
    street: str = ''


def number2kansuji(number_in_string: str) -> str:
    """

    Args:
        number_in_string:

    Returns:
    Example:
        >>> number2kansuji('北1条西12丁目')
        '北一条西十二丁目'
        >>> number2kansuji('北１条西１２丁目')
        '北一条西十二丁目'
    """
    number_in_string_sub = number_in_string
    for match in __number_pat.finditer(number_in_string):
        number = match.group()
        kansuji = int2kanji(int(number))
        split_string = list(number_in_string_sub)
        split_string[match.start():match.end()] = kansuji
        number_in_string_sub = ''.join(split_string)
    return number_in_string_sub


def _search(address: str, key2value: dict):
    matched = False
    for key in key2value.keys():
        matched = address.startswith(key)
        if matched:
            break
    return key, matched


def separate_address(address: str) -> ParsedAddress:
    """

    Args:
        address:

    Returns:

    Example:
        >>> separate_address('北海道札幌市中央区北1条西2丁目')
        ParsedAddress(prefecture='北海道', city='札幌市中央区', street='北一条西二丁目')
        >>> separate_address('奈良県高市郡高取町')
        ParsedAddress(prefecture='奈良県', city='高市郡高取町', street='')
    """
    address = address.strip()
    address = number2kansuji(address)
    parsed_address = ParsedAddress()

    if len(address) == 0:
        return parsed_address

    prefecture, matched_prefecture = _search(address, prefecture2city)
    # not found
    if not matched_prefecture:
        return parsed_address

    city2street = prefecture2city.get(prefecture)

    # remove a string of matching prefectures
    address = address.replace(prefecture, '')
    parsed_address.prefecture = prefecture

    city, matched_city = _search(address, city2street)
    # not found
    if not matched_city:
        return parsed_address

    streets = city2street.get(city)

    # remove a string of matching cities
    address = address.replace(city, '')
    parsed_address.city = city

    if len(address) == 0:
        return parsed_address

    matched_street = False
    for street in streets:
        matched_street = address.startswith(street)
        if matched_street:
            break

    # not found
    if not matched_street:
        return parsed_address

    # remove a string of matching cities
    address = address.replace(street, '')
    parsed_address.street = street

    return parsed_address



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['japanese_addresses']

package_data = \
{'': ['*']}

install_requires = \
['kanjize>=0.1.0,<0.2.0']

extras_require = \
{':python_version >= "3.6" and python_version < "3.7"': ['dataclasses>=0.7,<0.8']}

setup_kwargs = {
    'name': 'japanese-addresses',
    'version': '0.0.2',
    'description': '',
    'long_description': '# japanese-addresses\n\n[![PyPI version](https://badge.fury.io/py/japanese-addresses.svg)](https://badge.fury.io/py/japanese-addresses)\n[![Python package](https://github.com/wakamezake/japanese-addresses/workflows/Python%20package/badge.svg?branch=master)](https://github.com/wakamezake/japanese-addresses/actions?query=workflow%3A%22Python+package%22)\n[![codecov](https://codecov.io/gh/wakamezake/japanese-addresses/branch/master/graph/badge.svg)](https://codecov.io/gh/wakamezake/japanese-addresses)\n[![Binder](https://mybinder.org/badge.svg)](https://mybinder.org/v2/gh/wakamezake/japanese-addresses/master)\n\nParsing Japan addresses to prefectures and cities.\n\n## Installation\n\n```\npip install japanese-addresses\n```\n\n## Examples\n\n```python\nfrom japanese_addresses import separate_address\n\nparsed_address = separate_address(\'宮城県仙台市泉区市名坂字東裏97-1\')\n\nprint(parsed_address)\n"""\nParsedAddress(prefecture=\'宮城県\', city=\'仙台市泉区\', street=\'市名坂\')\n"""\n\nparsed_address = separate_address(\'鹿児島県志布志市志布志町志布志\')\n\nprint(parsed_address)\n"""\nParsedAddress(prefecture=\'鹿児島県\', city=\'志布志市\', street=\'志布志町志布志\')\n"""\n```\n\n## Testing\n\n```\npip install poetry\npoetry install\npoetry run pytest\n```\n\n## License\njapanese_addresses is licensed under [MIT](https://github.com/wakamezake/japanese-addresses/blob/master/LICENSE)\n',
    'author': 'wakame',
    'author_email': 'hotwater1367@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/wakamezake/japanese-addresses.git',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)
